SystemOrganization addCategory: #SeasideForSwazoo!

!SwazooServer class methodsFor: '*SeasideForSwazoo' stamp: 'pmm 8/31/2007 10:52'!
seasideStart
	self singleton seasideStart! !

!SwazooServer class methodsFor: '*SeasideForSwazoo' stamp: 'pmm 8/31/2007 10:53'!
seasideStop
	self singleton seasideStop! !

!SwazooServer methodsFor: '*SeasideForSwazoo' stamp: 'pmm 8/30/2007 12:54'!
seasideStart
	(self siteNamed: 'seasidedemo') isNil ifTrue:
		[SeasideSite newNamed: 'seasidedemo'.
		(SeasideSite named: 'seasidedemo') host: 'localhost' ip: '127.0.0.1'  port: 8888].
	self startSite: 'seasidedemo'! !

!SwazooServer methodsFor: '*SeasideForSwazoo' stamp: 'pmm 8/30/2007 12:56'!
seasideStop
	self stopSite: 'seasidedemo'! !

Site subclass: #SeasideSite
	instanceVariableNames: 'dispatcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SeasideForSwazoo'!

!SeasideSite commentStamp: 'pmm 9/19/2007 07:02' prior: 0!
I am a Seaside server adaptor for Swazoo 2. I offer the same functionality as WAKom which means to enocding support. Strings will be just byte arrays.

You can start and stop me with:
SwazooServer seasideStart
SwazooServer seasideStop
which listens to http://localhost:8888/seaside/

knowns bugs:
- error opens debugger!

!SeasideSite methodsFor: 'serving' stamp: 'pmm 9/10/2007 21:55'!
answerTo: aSwazooRequest
	| response |
	
	(self isImplemented: aSwazooRequest methodName)
		ifFalse: [ ^self notImplementedResponse ].

	response := [ self dispatcher handleRequest: (self convertRequest: aSwazooRequest) ]
		on: WADynamicVariable
		do: [ :notifiation |
			notifiation resume: notifiation defaultAction ].
	^ self convertResponse: response! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/29/2007 15:04'!
collectionClass
	^OrderedCollection! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/4/2007 20:46'!
convertHeader: aSwazooRequest
	| swazooFields fields |
	swazooFields := aSwazooRequest headers fields.
	fields := Dictionary new: swazooFields size.
	swazooFields keysAndValuesDo: [ :key :field |
		| value |
		value := String streamContents: [ :stream |
			field valuesAsStringOn: stream ].
		fields at: key asLowercase put: value ].
	^fields! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/4/2007 20:45'!
convertRequest: aSwazooRequest
	^WARequest
		method: aSwazooRequest methodName
		url: aSwazooRequest uriString
		headers: (self convertHeader: aSwazooRequest)
		fields: (self parseFields: aSwazooRequest)
		cookies: (self parseCookies: aSwazooRequest)
		nativeRequest: aSwazooRequest! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/1/2007 10:38'!
convertResponse: aSeasideResponse
	| swazooResponse |
	swazooResponse := HTTPResponse new.
			
	aSeasideResponse headers do: [ :each |
		| field fieldClass |
		fieldClass := HeaderField classForFieldName: each key.
		field := fieldClass newForFieldName: each key withValueFrom: each value.
		swazooResponse headers addField: field ].
	aSeasideResponse cookies do: [ :each | 
		swazooResponse cookie: (self cookieHeaderFor: each) ].
	swazooResponse code: aSeasideResponse status asNumber.

	swazooResponse entity: aSeasideResponse contents contents.
	swazooResponse contentType: aSeasideResponse contentType.

	aSeasideResponse release.
	^swazooResponse! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 8/31/2007 11:53'!
cookieHeaderFor: aCookie
	^String streamContents: [ :stream |
		stream
			nextPutAll: aCookie key;
			nextPutAll: '=';
			nextPutAll: (aCookie value ifNil: [ '' ]).
		aCookie path isNil ifFalse: [
			stream
				nextPutAll: '; path=';
				nextPutAll: aCookie path ].
		aCookie expiry isNil ifFalse: [
			stream
				nextPutAll: '; expires=';
				nextPutAll: aCookie expiryString ] ]! !

!SeasideSite methodsFor: 'accessing' stamp: 'pmm 8/30/2007 13:10'!
dispatcher
	^WADispatcher default! !

!SeasideSite methodsFor: 'serving-private' stamp: 'pmm 8/30/2007 12:52'!
helpResolve: aResolution 
	(self canAnswer and: [aResolution siteMatch: self]) ifFalse: [^nil].
	^self answerTo: aResolution request! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/4/2007 20:28'!
isImplemented: aMethodName
	^#('GET' 'POST') includes: aMethodName! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/4/2007 20:27'!
notImplementedResponse
	^HTTPResponse notImplemented! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/10/2007 21:55'!
parseCookies: aSwazooRequest
	| cookies swazooCookies cookieLine |
	cookies := Dictionary new.
	cookieLine := aSwazooRequest cookie.
	cookieLine isEmptyOrNil ifTrue: [ ^cookies ]. 
	swazooCookies := cookieLine findTokens: $;.
	swazooCookies do: [ :each |
		| pair key value |
		pair := each findTokens: $=.
		key := SpStringUtilities trimBlanksFrom: pair first.
		value := pair size > 1
			ifTrue: [ SpStringUtilities trimBlanksFrom:  pair second ]
			ifFalse: [ nil ].
		cookies at: key put: value ].
 	^cookies! !

!SeasideSite methodsFor: 'private' stamp: 'pmm 9/29/2007 15:22'!
parseFields: aSwazooRequest
	| fields |
	fields := Dictionary new.
	aSwazooRequest isPost
		ifTrue: [
			"break encapsulation"
			aSwazooRequest ensureFullRead.
			aSwazooRequest postData keysAndValuesDo: [ :key :value |
				| previousData data |
				data := value filename isEmptyOrNil
					ifTrue: [ value value ]
					ifFalse: [
						WAFile new
							fileName: value filename;
							contentType: value contentType;
							contents: value value;
							yourself ].
				previousData :=fields at: key ifAbsent: [ nil ].
				previousData isNil
					ifTrue: [ fields at: key put: data ]
					ifFalse: [
						(previousData isKindOf: self collectionClass)
							ifTrue: [ previousData add: data ]
							ifFalse: [ fields at: key put: (self collectionClass with: previousData with: data) ] ] ] ]
		ifFalse: [
			aSwazooRequest queries do: [ :each |
				fields at: each key put: each value ] ].
	^fields! !

SeasideSite subclass: #SeasideStreamSite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SeasideForSwazoo'!

!SeasideStreamSite commentStamp: 'pmm 9/18/2007 06:39' prior: 0!
I do not work yet, do not use me.!

!SeasideStreamSite methodsFor: 'accessing' stamp: 'pmm 9/1/2007 12:28'!
convertRequest: aSwazooRequest
	^(super convertRequest: aSwazooRequest)
		responseStream: aSwazooRequest parent stream;
		yourself! !
